;; jabber-ourversion.el.  Holds the version number in a format that
;; configure.ac can read.

;; On the following line, only change the part between double quotes:
(defconst jabber-version "0.8.92"
  "version returned to those who query us")

(provide 'jabber-ourversion)
